/* Copyright (C) 1998 Odysseas Pentakalos and Aram Khalili

  This program is freely distributable under the GNU General
  Public License Version 2 or later. The GNU GPL is available
  from the Free Software Foundation, Inc., 59 Temple Place,
  Suite 330, Boston, MA 02111-1307, USA.

  The authors can be contacted by email :
  Odysseas Pentakalos : odysseas@cs.umbc.edu
  Aram Khalili        : akhali1@gl.umbc.edu
*/

#include <stdio.h>
#include <string.h>
#include <scsi/sg.h>
#include <scsi/scsi.h>
#include <sys/types.h>
#include "tapelib.h"

int mode_select(int fd, __u16 save, elt_addr_assgn_t *eas, LCD_t *LCD, parity_t *par)
{
  /* structure for the MODE SELECT SCSI command (6-byte command) */

  typedef struct {
    struct sg_header head;
    u_char *cmd;
  } SCSI_Cmd_t;
  
  /* function pointers */
  
  void elt_addr_assgn(char *, elt_addr_assgn_t *);
  void trans_geo_desc(char *, tgd_t *);
  void dev_cap_p(char *, dev_cap_t *);
  void LCD_p(char *, LCD_t *);
  void parity(char *, parity_t *);
  
  SCSI_Cmd_t Cmd;
  int i, len=6, ret;
  unsigned char *buf, offset;

  /* blank command */

  if (eas)
    len+=20;
  
  if (LCD)
    len+=84;
  
  if (par)
    len+=4;
  
  if ((eas)||(LCD)||(par))
    len+=4;
  
  cmd = (u_char *) malloc(len);
  
  for (i=0; i<len; i++)
    Cmd.cmd[i]=0;
  
  Cmd.cmd[0]=MODE_SELECT; /* set command */
  Cmd.cmd[1]=0x10|save; /* LUN = 0, page format field (bit 4) = 1, reserved fields = 0,
			   save (bit 1) = save */
  Cmd.cmd[4]=len;

  offset=4;

  if (eas)
    {
      Cmd.cmd[10]=0x1d; /* pagecode */
      Cmd.cmd[11]=18;   /* page length   */
      Cmd.cmd[12]=TWOBYTEMSB(eas->tea);
      Cmd.cmd[13]=TWOBYTELSB(eas->tea);
      Cmd.cmd[14]=TWOBYTEMSB(eas->num_trans_elt);
      Cmd.cmd[15]=TWOBYTELSB(eas->num_trans_elt);
      Cmd.cmd[16]=TWOBYTEMSB(eas->sea);
      Cmd.cmd[17]=TWOBYTELSB(eas->sea);
      Cmd.cmd[18]=TWOBYTEMSB(eas->num_stor_elt);
      Cmd.cmd[19]=TWOBYTELSB(eas->num_stor_elt);
      Cmd.cmd[20]=TWOBYTEMSB(eas->iea);
      Cmd.cmd[21]=TWOBYTELSB(eas->iea);
      Cmd.cmd[22]=TWOBYTEMSB(eas->num_imp_elt);
      Cmd.cmd[23]=TWOBYTELSB(eas->num_imp_elt);
      Cmd.cmd[24]=TWOBYTEMSB(eas->dea);
      Cmd.cmd[25]=TWOBYTELSB(eas->dea);
      Cmd.cmd[26]=TWOBYTEMSB(eas->num_data_elt);
      Cmd.cmd[27]=TWOBYTELSB(eas->num_data_elt);
      
      offset += 20;
    }

  if(LCD)
    {
      Cmd.cmd[6+offset+0]=0x22; /* pagecode */
      Cmd.cmd[6+offset+1]=0x52; /* page length */
      Cmd.cmd[6+offset+2]=0x80|(LCD->LCDsec<<6)|(LCD->wl1<<3)|(LCD->wl2<<2)|
	                       (LCD->wl3<<1)|LCD->wl4;

      for (i=0;i<20;i++)
	Cmd.cmd[6+offset+4+i]=LCD->line1[i];
      for (i=0;i<20;i++)
	Cmd.cmd[6+offset+24+i]=LCD->line2[i];
      for (i=0;i<20;i++)
	Cmd.cmd[6+offset+44+i]=LCD->line3[i];
      for (i=0;i<20;i++)
	Cmd.cmd[6+offset+64+i]=LCD->line4[i];

      offset+=84;
    }

  if(par)
    {
      Cmd.cmd[6+offset+0]=0; /* LUN = 0, pagecode = 0 */
      Cmd.cmd[6+offset+1]=2; /* page length */
      Cmd.cmd[6+offset+2]= par->par? 32 :0;
      Cmd.cmd[6+offset+3]=par->retries;
    }
  
  /* Prepare the SCSI command header */
  Cmd.head.pack_len=sizeof(struct sg_header)+len;
  Cmd.head.reply_len=sizeof(struct sg_header)+6;
  
#ifdef DEBUG
  printf("mode_select: Sending out the MODE_SELECT command\n");
  printf("mode_select: eas: %d, LCD: %d, par: %d\n",(eas)?1:0,LCD?1:0,par?1:0);
#endif DEBUG

  ret=write(fd, &Cmd, Cmd.head.pack_len);

#ifdef DEBUG
  printf("mode_select: The write returned %d status\n", ret);
#endif DEBUG

  /* Now lets read back the results */

  ret=read(fd, &Cmd, Cmd.head.reply_len);

#ifdef DEBUG
  printf("move_medium: The read returned %d bytes (expected %d)\n", ret, Cmd.head.reply_len);
#endif DEBUG

return ret;

}
