/* Copyright (C) 1998 Odysseas Pentakalos and Aram Khalili

  This program is freely distributable under the GNU General
  Public License Version 2 or later. The GNU GPL is available
  from the Free Software Foundation, Inc., 59 Temple Place,
  Suite 330, Boston, MA 02111-1307, USA.

  The authors can be contacted by email :
  Odysseas Pentakalos : odysseas@cs.umbc.edu
  Aram Khalili        : akhali1@gl.umbc.edu
*/

#include <stdio.h>
#include <string.h>
#include <scsi/sg.h>
#include <scsi/scsi.h>
#include <sys/types.h>
#include "tapelib.h"

int move_medium(int fd, __u16 tea, __u16 src, __u16 dest)
{
  typedef struct {
    struct sg_header head;
    u_char cmd[12];
  } SCSI_Cmd_t;

  SCSI_Cmd_t Cmd;
  int i, ret;

  for (i=0; i<12; i++)
    Cmd.cmd[i]=0;

  Cmd.cmd[0]=MOVE_MEDIUM;
  Cmd.cmd[1]=0x00;

  /* 
     Transport element address that will perform the move operation
     */
  Cmd.cmd[2]=tea >> 8;
  Cmd.cmd[3]=tea & 0x00ff;

  /*
    Source element address from where cartridge will be taken
    */
  Cmd.cmd[4]=src >> 8;
  Cmd.cmd[5]=src & 0x00ff;

  /*
    Destination element address to where cartridge will be taken
    */
  Cmd.cmd[6]=dest >> 8;
  Cmd.cmd[7]=dest & 0x00ff;

  /*
    Reserved bytes should all be zero-ed
    */
  Cmd.cmd[8]=0;
  Cmd.cmd[9]=0;
  Cmd.cmd[10]=0;
  Cmd.cmd[11]=0;

  /*
    Prepare the header
    */
  Cmd.head.pack_len=sizeof(Cmd);
  Cmd.head.reply_len=sizeof(Cmd);
  Cmd.head.pack_id=1;

#ifdef DEBUG
  printf("move_medium: Sending out the MOVE_MEDIUM command\n");
  printf("move_medium: using transport element %d to go from %d to %d\n",
	 tea, src, dest);
#endif DEBUG

  ret=write(fd, &Cmd, sizeof(Cmd));

#ifdef DEBUG
  printf("move_medium: The write returned %d status\n", ret);
#endif DEBUG

  /*
    Now lets read back the results
    */
  ret=read(fd, &Cmd, sizeof(Cmd));

#ifdef DEBUG
  printf("move_medium: The read returned %d bytes\n", ret);
#endif DEBUG

  return ret;

}













