/* Copyright (C) 1998 Odysseas Pentakalos and Aram Khalili

  This program is freely distributable under the GNU General
  Public License Version 2 or later. The GNU GPL is available
  from the Free Software Foundation, Inc., 59 Temple Place,
  Suite 330, Boston, MA 02111-1307, USA.

  The authors can be contacted by email :
  Odysseas Pentakalos : odysseas@cs.umbc.edu
  Aram Khalili        : akhali1@gl.umbc.edu
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include "tapelib.h"

main(int argc, char *argv[])
{
  int i, fd, pagecode =-1, pagecontrol=-1;

  elt_addr_assgn_t *eas;
  tgd_t *tgd;
  dev_cap_t *dct;
  LCD_t *LCD;
  parity_t *par;

  if (argc < 4) {
    fprintf(stderr, "Usage: %s dev pagecode pagecontrol\n", argv[0]);
    exit(1);
  }

  pagecode = atoi(argv[2]);
  pagecontrol = atoi(argv[3]);
  if((pagecontrol<0)||(pagecontrol>3))
    {
      fprintf(stderr,"%s: pagecontrol invalid\n", argv[0]);
      fprintf(stderr, "Usage: %s dev pagecode pagecontrol\n", argv[0]);
      exit(0);
    }

#ifdef DEBUG
  printf("pagecode = %d; pagecontrol = %d\n", pagecode, pagecontrol);
#endif

  switch(pagecode){
  case 0x1D: eas = (elt_addr_assgn_t *)malloc(sizeof(elt_addr_assgn_t *)); 
#ifdef DEBUG
             printf("elt addr assgn page\n");
#endif
	     break;
  case 0x1E: tgd = (tgd_t*) malloc(sizeof(tgd_t));
#ifdef DEBUG
             printf("tgd page\n");
#endif
	     break;
  case 0x1F: dct = (dev_cap_t *) malloc (sizeof(dev_cap_t));
#ifdef DEBUG
             printf("dct page\n");
#endif
	     break;
  case 0x22: LCD = (LCD_t*)malloc(sizeof(LCD_t));
#ifdef DEBUG
             printf("LCD page\n");
#endif
	     break;
  case 0x00: par = (parity_t*) malloc(sizeof(parity_t));
#ifdef DEBUG
             printf("parity page\n");
#endif
	     break;
  case 0x3F: eas = (elt_addr_assgn_t *)malloc(sizeof(elt_addr_assgn_t *));
             tgd = (tgd_t*) malloc(sizeof(tgd_t));
	     dct = (dev_cap_t *) malloc (sizeof(dev_cap_t));
	     LCD = (LCD_t*)malloc(sizeof(LCD_t));
	     par = (parity_t*) malloc(sizeof(parity_t));
#ifdef DEBUG
             printf("all pages\n");
#endif
             break;
  default: /* invalid */
             fprintf(stderr, "%s: Invalid pagecode\n", argv[0]);
	     return 0;
  }

  if ((fd=open(argv[1], O_RDWR | O_EXCL)) == -1) {
    fprintf(stderr, "%s: could not open %s in O_RDWR | O_EXCL mode\n",
	    argv[0], argv[1]);
    exit(1);
  }

  mode_sense(fd, (__u16) atoi(argv[3]), (__u16) atoi(argv[2]), eas, tgd, dct, LCD, par);

  if((pagecode==0x1d)||(pagecode==0x3f))
    {
      printf("\nElement Address Assignment Page\nSalvable = %s;\ntea=%d; no.=%d;\nsea=%d; no.=%d;\niea=%d; no.=%d\ndea=%d; no.=%d\n",
	     eas->ps?"true":"false",eas->tea,eas->num_trans_elt, eas->sea,eas->num_stor_elt,
	     eas->iea,eas->num_imp_elt,eas->dea,eas->num_data_elt);
    }
    
  if((pagecode==0x1e)||(pagecode==0x3f))
    {
      printf("\nTransport Geometry Descriptor Page\nSalvable %s;\nrotate=%d\nmember offset=%d\n",tgd->ps ? "true" : "false",
	     tgd->rot,
	     tgd->trans_elt_no);
    }
  

  if((pagecode==0x1f)||(pagecode==0x3f))
    {
      printf("\nDevice Capabilities Page\nSalvable = %s\n",dct->ps?"true":"false");
      printf("data transfer bit vector = %d\nimport/export bit vector = %d\nstorage bit vector = %d\nmedium transport bit vector = %d\n", dct->dt, dct->ie, dct->st, dct->mt);
    }

  if((pagecode==0x22)||(pagecode==0x3f))
    {
      printf("\nLCD Mode Page\nSalvable = %s\nsecurity is %s\n", LCD->ps ? "true" : "false",LCD->LCD_sec ? "off" : "on");
      printf("Line1: ");
      for(i=0;i<20;i++)
	putchar(LCD->line1[i]);
      putchar('\n');
      printf("Line2: ");
      for(i=0;i<20;i++)
	putchar(LCD->line2[i]);
      putchar('\n');
      printf("Line3: ");
      for(i=0;i<20;i++)
	putchar(LCD->line3[i]);
      putchar('\n');
      printf("Line4: ");
      for(i=0;i<20;i++)
	putchar(LCD->line4[i]);
      putchar('\n');
    }

  if((pagecode==0x00)||(pagecode==0x3f))
    {
      printf("\nParity Page\nSalvable = %s\nParity is %s\nRetries = %d\n",par->ps ? "true" : "false", par->par ? "disabled" : "enabled", par->retries);
    }


  exit(0);

}









